from os import environ
from flask import Flask
from flask_sqlalchemy import SQLAlchemy


def create_app(test_config=None):
    
    #create and configure the app
    app = Flask(__name__, instance_relative_config=True)
    app.secret_key = environ.get('SECRET_KEY')
    
    app.config.from_mapping(
    SQLALCHEMY_DATABASE_URI=environ.get('SQLALCHEMY_DATABASE_URI'),
    SQLALCHEMY_TRACK_MODIFICATIONS = False,
    )
    db = SQLAlchemy(app)
    db.init_app(app)

    app.app_context().push()

    if test_config is None:
        # load the instance config, if it exists, when not testing
        app.config.from_pyfile('config.py', silent=True)
    else:
        # load the test config if passed in
        app.config.from_mapping(test_config)


    # Flask commands
    from .database import db_commands
    db_commands(app)

    # Web Routes
    from .routes import webRoutes
    app.register_blueprint(webRoutes.web)

    # API Routes
    from .routes import apiRoutes
    app.register_blueprint(apiRoutes.api, url_prefix='/api')

    # Jinja env variables
    app.jinja_env.globals['ATOMO4_URL'] = environ.get('ATOMO4_URL')

    return app