from flask import current_app, request, jsonify, make_response
from thermodynamics.mix.equilibrium import solve_eos, bubble_temperature, bubble_pressure, dew_temperature, dew_pressure
from thermodynamics.mix.binaryDiagrams import pressure_composition, temperature_composition
from thermodynamics.mix.diagrams import pt
from ..helpers.substanceHelpers import getSubstanceData
from numpy import round, zeros

def solve_bubble_temperature():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']
    new_t, p, liq_compositions, new_vap_compositions = bubble_temperature(temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function)

    result = solve_eos(
        new_t,
        pressure,
        tc,
        pc,
        acentric,
        liq_compositions,
        new_vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (temperature == new_t):
        response = {"message":"Set a new temperature approximation"}
    else :
        response = {
            "liquid" : {
                "phase":"Liquid",
                "fugacity":list(round(result['liq_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_liq'], 2)),
                "entropy":list(round(result['entropy_liq'], 2)),
                "z":float(round(result['z_liq'], 4)),
                "compositions": list(round(liq_compositions, 4)),
                "temperature" : float(round(new_t, 4)),
                "pressure":float(round(p, 4)),
                "substances":substances
            },
            "vapor" : {
                "phase":"Vapor",
                "fugacity":list(round(result['vap_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_vap'], 2)),
                "entropy":list(round(result['entropy_vap'], 2)),
                "z":float(round(result['z_vap'], 4)),
                "compositions": list(round(new_vap_compositions, 4)),
                "temperature" : float(round(new_t, 4)),
                "pressure":float(round(p, 4)),
                "substances":substances
            }
        }

    res = make_response(jsonify(response), 200)

    return res

def solve_bubble_pressure():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']
    t,new_p,liq_compositions,new_vap_compositions = bubble_pressure(temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function)
    
    result = solve_eos(
        t,
        new_p,
        tc,
        pc,
        acentric,
        liq_compositions,
        new_vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (pressure == new_p):
        response = {"message":"Set a new pressure approximation"}
    else :
        response = {
            "liquid" : {
                "phase":"Liquid",
                "fugacity":list(round(result['liq_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_liq'], 2)),
                "entropy":list(round(result['entropy_liq'], 2)),
                "z":float(round(result['z_liq'], 4)),
                "compositions": list(round(liq_compositions, 4)),
                "temperature" : float(round(t, 4)),
                "pressure":float(round(new_p, 4)),
                "substances":substances
            },
            "vapor" : {
                "phase":"Vapor",
                "fugacity":list(round(result['vap_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_vap'], 2)),
                "entropy":list(round(result['entropy_vap'], 2)),
                "z":float(round(result['z_vap'], 4)),
                "compositions": list(round(new_vap_compositions, 4)),
                "temperature" : float(round(t, 4)),
                "pressure":float(round(new_p, 4)),
                "substances":substances
            }
        }

    res = make_response(jsonify(response), 200)

    return res

def solve_dew_temperature():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']

    new_t,p,new_liq_compositions,vap_compositions = dew_temperature(temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function)

    result = solve_eos(
        new_t,
        p,
        tc,
        pc,
        acentric,
        new_liq_compositions,
        vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (temperature == new_t):
        response = {"message":"Set a new temperature approximation"}
    else :
        response = {
            "liquid" : {
                "phase":"Liquid",
                "fugacity":list(round(result['liq_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_liq'], 2)),
                "entropy":list(round(result['entropy_liq'], 2)),
                "z":float(round(result['z_liq'], 4)),
                "compositions": list(round(new_liq_compositions, 4)),
                "temperature" : float(round(new_t, 4)),
                "pressure":float(round(p, 4)),
                "substances":substances
            },
            "vapor" : {
                "phase":"Vapor",
                "fugacity":list(round(result['vap_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_vap'], 2)),
                "entropy":list(round(result['entropy_vap'], 2)),
                "z":float(round(result['z_vap'], 4)),
                "compositions": list(round(vap_compositions, 4)),
                "temperature" : float(round(new_t, 4)),
                "pressure":float(round(p, 4)),
                "substances":substances
            }
        }

    res = make_response(jsonify(response), 200)

    return res

def solve_dew_pressure():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']

    t,new_p,new_liq_compositions,vap_compositions = dew_pressure(temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function)

    result = solve_eos(
        t,
        new_p,
        tc,
        pc,
        acentric,
        new_liq_compositions,
        vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (pressure == new_p):
        response = {"message":"Set a new pressure approximation"}
    else :
        response = {
            "liquid" : {
                "phase":"Liquid",
                "fugacity":list(round(result['liq_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_liq'], 2)),
                "entropy":list(round(result['entropy_liq'], 2)),
                "z":float(round(result['z_liq'], 4)),
                "compositions": list(round(new_liq_compositions, 4)),
                "temperature" : float(round(t, 4)),
                "pressure":float(round(new_p, 4)),
                "substances":substances
            },
            "vapor" : {
                "phase":"Vapor",
                "fugacity":list(round(result['vap_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_vap'], 2)),
                "entropy":list(round(result['entropy_vap'], 2)),
                "z":float(round(result['z_vap'], 4)),
                "compositions": list(round(vap_compositions, 4)),
                "temperature" : float(round(t, 4)),
                "pressure":float(round(new_p, 4)),
                "substances":substances
            }
        }

    res = make_response(jsonify(response), 200)

    return res

def get_input_data(req):
    temperature = float(req['temperature']) if req.get('temperature') else None
    pressure = float(req['pressure']) if req.get('pressure') else None
    method = req['eos']
    alfa_function = req['alfa']
    compositions = [float(element) for element in req['compositions']]

    substances = []
    critical_temperatures = []
    cirtical_pressures = []
    acentric_factors = []
    heat_capacities_constants = []

    for substance_id in req['substances']:
        substance, tc, pc, acentric, heat_capacity_constants = getSubstanceData(substance_id)
        substances.append(substance.name)
        critical_temperatures.append(tc)
        cirtical_pressures.append(pc)
        acentric_factors.append(acentric)
        heat_capacities_constants.append(heat_capacity_constants)

    n = len(req['substances'])
    kij = []
    if(len(req['kij'])):
        for i in range(0,len(req['kij']), n):
            aux = [float(element) for element in req['kij'][i:i+n]]
            kij.append(aux)
    else:
        for i in range(0,n):
            kij.append(zeros(n))

    return (temperature, pressure, critical_temperatures, cirtical_pressures, acentric_factors, compositions, compositions, kij, heat_capacities_constants, method, alfa_function, substances)

def pressure_temperature_diagram():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']
    bubble_line, dew_line = pt(tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function, atomo4=True)

    response = {
        "bubble_line" : bubble_line,
        "dew_line" : dew_line
    }

    res = make_response(jsonify(response), 200)
    return res
