from flask import current_app, request, jsonify, make_response
from thermodynamics.pure.equilibrium import solve_eos, solve_VLE
from thermodynamics.pure.diagrams import pv, tv, pvt
from numpy import log10
from ..helpers.substanceHelpers import getSubstanceData

def solve():
    req = request.get_json()
    temperature = float(req['temperature'])
    pressure = float(req['pressure'])
    method = req['eos']
    alfa_function = req['alfa']
    substance, tc, pc, acentric, heat_capacity_constants = getSubstanceData(req['selected_substance'])
    
    result = solve_eos(temperature,pressure,tc,pc,acentric,method=method,alfa_function=alfa_function,properties=True,heat_capacity=heat_capacity_constants)
    result['vap_fugacity'] = round(result['vap_fugacity'], 4)
    result['enthalpy_vap'] = round(result['enthalpy_vap'], 4)
    result['entropy_vap'] = round(result['entropy_vap'], 4)
    result['liq_fugacity'] = round(result['liq_fugacity'], 4)
    result['enthalpy_liq'] = round(result['enthalpy_liq'], 4)
    result['entropy_liq'] = round(result['entropy_liq'], 4)
    result['z_vap'] = round(result['z_vap'], 4)
    result['z_liq'] = round(result['z_liq'], 4)
    result['temperature'] = round(temperature, 4)
    result['pressure'] = round(pressure, 4)

    res = make_response(jsonify(result), 200)
    
    return res

def equilibrium():
    req = request.get_json()
    temperature = float(req['temperature'])
    pressure = float(req['pressure'])
    method = req['eos']
    alfa_function = req['alfa']
    substance, tc, pc, acentric, heat_capacity_constants = getSubstanceData(req['selected_substance'])

    if(req['equilirbium']=='temperature'):
        temperature = solve_VLE(temperature, pressure, tc, pc, acentric, solving_for='temperature')
    elif(req['equilirbium']=='pressure'):
        pressure = solve_VLE(temperature, pressure, tc, pc, acentric, solving_for='pressure')
    
    result = solve_eos(temperature,pressure,tc,pc,acentric,method=method,alfa_function=alfa_function,properties=True,heat_capacity=heat_capacity_constants)
    result['vap_fugacity'] = round(result['vap_fugacity'], 4)
    result['enthalpy_vap'] = round(result['enthalpy_vap'], 4)
    result['entropy_vap'] = round(result['entropy_vap'], 4)
    result['liq_fugacity'] = round(result['liq_fugacity'], 4)
    result['enthalpy_liq'] = round(result['enthalpy_liq'], 4)
    result['entropy_liq'] = round(result['entropy_liq'], 4)
    result['z_vap'] = round(result['z_vap'], 4)
    result['z_liq'] = round(result['z_liq'], 4)
    result['temperature'] = round(temperature, 2)
    result['pressure'] = round(pressure, 2)
    res = make_response(jsonify(result), 200)
    
    return res



def pressureVolumeDiagram():
    req = request.get_json()
    temperature = float(req['temperature'])
    method = req['eos']
    alfa_function = req['alfa']
    substance, tc, pc, acentric, heat_capacity_constants = getSubstanceData(req['selected_substance'])

    pressures, volumes = pv(temperature, tc, pc, acentric,method=method, alfa_function=alfa_function,pvt=True)
    data={
        'pressures':list(pressures),
        'volumes':list(log10(volumes))
    }

    res = make_response(jsonify(data), 200)
    return res

def temperatureVolumeDiagram():
    req = request.get_json()
    pressure = float(req['pressure'])
    method = req['eos']
    alfa_function = req['alfa']
    substance, tc, pc, acentric, heat_capacity_constants = getSubstanceData(req['selected_substance'])

    temperatures, volumes = tv(pressure, tc, pc, acentric, method=method, alfa_function=alfa_function,atomo4=True)
    data = {
        'temperatures':list(temperatures),
        'volumes':list(log10(volumes))
    }

    res = make_response(jsonify(data), 200)
    return res

def pressureVolumeTemperatureDiagram():
    req = request.get_json()
    method = req['eos']
    alfa_function = req['alfa']
    substance, tc, pc, acentric, heat_capacity_constants = getSubstanceData(req['selected_substance'])

    volumes, temperatures, pressures = pvt(tc, pc, acentric, method=method, alfa_function=alfa_function, atomo4=True)
    data = {
        'pressures':list(pressures),
        'volumes':list(volumes),
        'temperatures':list(temperatures)
    }

    res = make_response(jsonify(data), 200)
    return res
