'use strict';

import { API_URL } from "../../constants";

class BinaryMixture {

    getInputData () {
        const first_substance = document.getElementById('first_substance').value;
        const second_substance = document.getElementById('second_substance').value;
        const temperature = document.getElementById('temperature').value;
        const pressure = document.getElementById('pressure').value;
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;
        const first_composition = document.getElementById('first_composition').value;
        const second_composition = document.getElementById('second_composition').value;
        const liq_phase = document.getElementById('liq_phase').checked;
        const vap_phase = document.getElementById('vap_phase').checked;
        const k12 = document.getElementById('k12').value;
        const k21 = document.getElementById('k21').value;

        let phase;
        if (liq_phase) {
            phase = 'liq';
        } else if (vap_phase) {
            phase = 'vap';
        }

        return {first_substance, second_substance, temperature, pressure, eos, alfa, first_composition, second_composition, phase, k12, k21};
    }

    async solve_eos() {
        const url = API_URL + '/binary-mixture/solve';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        this.renderSolution(body);
    }

    async bubble_temperature() {
        const url = API_URL + '/binary-mixture/bubble-temperature';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        await this.handleResponse(response, "Set a new initial temperature value for equilibrium calculations.");
    }

    async bubble_pressure() {
        const url = API_URL + '/binary-mixture/bubble-pressure';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        await this.handleResponse(response, "Set a new initial pressure value for equilibrium calculations.");
    }

    async dew_temperature() {
        const url = API_URL + '/binary-mixture/dew-temperature';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        await this.handleResponse(response, "Set a new initial temperature value for equilibrium calculations.");
    }

    async dew_pressure() {
        const url = API_URL + '/binary-mixture/dew-pressure';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });
        await this.handleResponse(response, "Set a new initial pressure value for equilibrium calculations.");
    }

    async handleResponse(response, warningMessage) {
        try {
            const body = await response.json();
            if (body.message) {
                this.renderWarning(body.message);
                return;
            }
            this.renderSolution(body.liquid);
            this.renderSolution(body.vapor);

        } catch (error) {
            this.renderWarning(warningMessage)
            return;
        }
    }

    calculate_second_composition() {
        const first_composition_ele = document.getElementById('first_composition');
        const first_composition = parseFloat(first_composition_ele.value).toFixed(4);
        const error_message = document.getElementById('error_message');
        const second_composition = document.getElementById('second_composition');
        const liq_phase = document.getElementById('liq_phase').checked;
        const vap_phase = document.getElementById('vap_phase').checked;

        if (first_composition < 0 || first_composition > 1) {
            error_message.innerText = "Composition must be between 0 and 1";
            error_message.classList.remove('d-none');
            this.setAvailableButtons(null, null, true);
            second_composition.value = null;
        } else {
            error_message.classList.add('d-none');
            first_composition_ele.value = first_composition;
            second_composition.value = (1 - first_composition).toFixed(4);
            this.setAvailableButtons(liq_phase, vap_phase);
        }

        return;
    }

    setAvailableButtons(liq, vap, all=false) {
        const bubble_t_btn = document.getElementById('bubble_temperature');
        const bubble_p_btn = document.getElementById('bubble_pressure');
        const dew_t_btn = document.getElementById('dew_temperature');
        const dew_p_btn = document.getElementById('dew_pressure');

        if (all){
            dew_t_btn.disabled = true;
            dew_p_btn.disabled = true;
            bubble_t_btn.disabled = true;
            bubble_p_btn.disabled = true;
        } else if (liq) {
            dew_t_btn.disabled = true;
            dew_p_btn.disabled = true;
            bubble_t_btn.disabled = false;
            bubble_p_btn.disabled = false;
        } else if (vap) {
            dew_t_btn.disabled = false;
            dew_p_btn.disabled = false;
            bubble_t_btn.disabled = true;
            bubble_p_btn.disabled = true;
        } else {
            dew_t_btn.disabled = true;
            dew_p_btn.disabled = true;
            bubble_t_btn.disabled = true;
            bubble_p_btn.disabled = true;
        }
    }

    async configureButtons() {
        const liq_phase = document.getElementById('liq_phase').checked;
        const vap_phase = document.getElementById('vap_phase').checked;

        this.setAvailableButtons(liq_phase, vap_phase);
    }

    add_kij() {
        const del_btn = document.getElementById('del_kij');
        const kij_form = document.getElementById('binary_interaction_form');

        this.classList.add('d-none');
        del_btn.classList.remove('d-none');
        kij_form.classList.remove('d-none');
    }

    del_kij() {
        const add_btn = document.getElementById('add_kij');
        const kij_form = document.getElementById('binary_interaction_form');
        const k12_ele = document.getElementById('k12');
        const k21_ele = document.getElementById('k21');

        k12_ele.value = null;
        k21_ele.value = null;
        k12_ele.placeholder = "0.0";
        k21_ele.placeholder = "0.0";
        this.classList.add('d-none');
        add_btn.classList.remove('d-none');
        kij_form.classList.add('d-none');
    }

    renderSolution(response) {
        const warning_div = document.getElementById('warning');
        if (!warning_div.classList.contains('d-none')) {
            warning_div.classList.add('d-none');
        }

        const {solution_ele, title_ele, first_substance, first_composition, first_fugacity, first_enthalpy, first_entropy,second_substance, second_composition, second_fugacity, second_enthalpy, second_entropy} = this.getSolutionElements(response.phase);

        title_ele.innerText = response.phase + ' phase at ' + response.temperature + ' K and ' + response.pressure + ' bar';
        first_substance.innerText = response.substances[0];
        first_composition.innerText = response.compositions[0];
        first_fugacity.innerText = response.fugacity[0];
        first_enthalpy.innerText = response.enthalpy[0];
        first_entropy.innerText = response.entropy[0];

        second_substance.innerText = response.substances[1];
        second_composition.innerText = response.compositions[1];
        second_fugacity.innerText = response.fugacity[1];
        second_enthalpy.innerText = response.enthalpy[1];
        second_entropy.innerText = response.entropy[1];
        
        solution_ele.classList.remove('d-none');
    }

    renderWarning(message) {
        const warning_div = document.getElementById('warning');
        const warninig_text = document.getElementById('warnin_text');
        const liq_solution = document.getElementById('liq_solution');
        const vap_solution = document.getElementById('vap_solution');
        
        if (!liq_solution.classList.contains('d-none')) {
            liq_solution.classList.add('d-none');
        }

        if (!vap_solution.classList.contains('d-none')) {
            vap_solution.classList.add('d-none');
        }

        warning_div.classList.remove('d-none');
        warninig_text.innerText = message;
    }

    getSolutionElements(phase) {
        let prefix;
        if (phase == 'Liquid') {
            prefix = 'liq';
        } else {
            prefix = 'vap';
        }

        const solution_ele = document.getElementById(prefix + '_solution');
        const title_ele = document.getElementById(prefix + '_phase_title');
        const first_substance = document.getElementById(prefix + '_first_substance_title');
        const first_composition = document.getElementById(prefix + '_first_composition');
        const first_fugacity = document.getElementById(prefix + '_first_fugacity');
        const first_enthalpy = document.getElementById(prefix + '_first_enthalpy');
        const first_entropy = document.getElementById(prefix + '_first_entropy');
        const second_substance = document.getElementById(prefix + '_second_substance_title');
        const second_composition = document.getElementById(prefix + '_second_composition');
        const second_fugacity = document.getElementById(prefix + '_second_fugacity');
        const second_enthalpy = document.getElementById(prefix +'_second_enthalpy');
        const second_entropy = document.getElementById(prefix + '_second_entropy');

        return {solution_ele, title_ele, first_substance, first_composition, first_fugacity, first_enthalpy, first_entropy,second_substance, second_composition, second_fugacity, second_enthalpy, second_entropy};
    }

    configureAjaxCalls() {
        //document.getElementById('calculate_btn').addEventListener('click', this.solve_eos.bind(this));
        document.getElementById('bubble_temperature').addEventListener('click', this.bubble_temperature.bind(this));
        document.getElementById('bubble_pressure').addEventListener('click', this.bubble_pressure.bind(this));
        document.getElementById('dew_temperature').addEventListener('click', this.dew_temperature.bind(this));
        document.getElementById('dew_pressure').addEventListener('click', this.dew_pressure.bind(this));
        document.getElementById('first_composition').addEventListener('change', this.calculate_second_composition.bind(this));
        document.getElementById('liq_phase').addEventListener('change', this.configureButtons.bind(this));
        document.getElementById('vap_phase').addEventListener('change', this.configureButtons.bind(this));
        document.getElementById('add_kij').addEventListener('click', this.add_kij);
        document.getElementById('del_kij').addEventListener('click', this.del_kij);
    }
}

export const binary_mixture = new BinaryMixture();
