'use strict';

import { API_URL } from "../../constants";
import Plotly from 'plotly.js-basic-dist-min';

class PressureCompositionDiagram {

    getInputData() {
        const first_substance = document.getElementById('first_substance').value;
        const second_substance = document.getElementById('second_substance').value;
        const temperature = document.getElementById('temperature').value;
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;
        const k12 = document.getElementById('k12').value;
        const k21 = document.getElementById('k21').value;

        return {first_substance, second_substance, temperature, eos, alfa, k12, k21};
    }

    async render() {
        const url = API_URL + '/binary-mixture/pxy';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        const solution_ele = document.getElementById('solution');
        const card_title = document.getElementById('solution_title');
        card_title.innerText = 'Pressure composition diagram at ' + data.temperature + ' K';
        const bubble_line = {
            x: body.bubble_line[0],
            y: body.bubble_line[1],
            mode: 'markers',
            type: 'scatter',
            name: 'Bubble line'
        };

        const dew_line = {
            x: body.dew_line[0],
            y: body.dew_line[1],
            mode: 'markers',
            type: 'scatter',
            name: 'Dew line'
        }

        const points = [bubble_line, dew_line];
        const layout = {
            title: 'Pressure composition diagram at ' + data.temperature + ' K',
            xaxis: {
                title: {
                    text: 'x, y',
                }
            },
            yaxis: {
                title: {
                    text: 'Pressure [bar]',
                }
            }
        }

        const config = { responsive: true }

        Plotly.newPlot('pxy_diagram', points, layout, config);
        solution_ele.classList.remove('d-none');
    }

    async download_pxy_csv() {
        const url = API_URL + '/binary-mixture/pxy';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        let csvContent = "data:text/csv;charset=utf-8,composition,bubble_pressure,dew_pressure\r\n";

        for (let i = 0; i < body.bubble_line[0].length; i++) {
            const composition = body.bubble_line[0][i];
            const bubble_pressure = body.bubble_line[1][i];
            const dew_pressure = body.dew_line[1][i];

            csvContent += await composition + ',' + bubble_pressure + ',' + dew_pressure + "\r\n";
            
        }

        const encodedUri = encodeURI(csvContent);
        const link = document.createElement("a");
        link.setAttribute("href", encodedUri);
        link.setAttribute("download", 'Pxy at ' + data.temperature + ' K.csv');
        document.body.appendChild(link); // Required for FF

        link.click();
    }

    add_kij() {
        const del_btn = document.getElementById('del_kij');
        const kij_form = document.getElementById('binary_interaction_form');

        this.classList.add('d-none');
        del_btn.classList.remove('d-none');
        kij_form.classList.remove('d-none');
    }

    del_kij() {
        const add_btn = document.getElementById('add_kij');
        const kij_form = document.getElementById('binary_interaction_form');
        const k12_ele = document.getElementById('k12');
        const k21_ele = document.getElementById('k21');

        k12_ele.value = null;
        k21_ele.value = null;
        k12_ele.placeholder = "0.0";
        k21_ele.placeholder = "0.0";
        this.classList.add('d-none');
        add_btn.classList.remove('d-none');
        kij_form.classList.add('d-none');
    }

    configureAjaxCalls() {
        document.getElementById('draw_binary_p_xy').addEventListener('click', this.render.bind(this));
        document.getElementById('add_kij').addEventListener('click', this.add_kij);
        document.getElementById('del_kij').addEventListener('click', this.del_kij);
        document.getElementById('download_pxy_csv').addEventListener('click', this.download_pxy_csv.bind(this));
    }
}

export const pressure_composition_diagram = new PressureCompositionDiagram();
