'use strict';

import { API_URL } from "../../constants";

class Mixture {
    constructor() {
        this.extra_substance = 2;
    }

    addSubstance() {
        const substances_div = document.getElementById('substances_element');
        const substances_dropdown = document.getElementById('substances_dropdown');
        const molar_compositions_div = document.getElementById('molar_compositions_element');
        const new_dropdown = substances_dropdown.cloneNode(true);

        this.extra_substance += 1;

        // Substance selection
        new_dropdown.id = null;
        new_dropdown.classList.remove('d-none');
        new_dropdown.firstElementChild.name = 'substances[]';
        new_dropdown.firstElementChild.classList.add('new-select2');
        substances_div.appendChild(new_dropdown);
        $('.new-select2').select2({
            theme:"bootstrap4"
        });

        // Molar composition
        const div = document.createElement('div');
        const input = document.createElement('input');
        const del_button = document.createElement('button');
        const trash_icon = document.createElement('i');

        div.classList.add('d-flex', 'justify-content-between');
        input.classList.add('form-control', 'mb-2');
        input.setAttribute("type", "number");
        input.setAttribute("min","0");
        input.setAttribute("max", "1");
        input.setAttribute("name", "molar_compositions[]");
        input.setAttribute("placeholder", "Molar composition");
        input.style.width = "90%";
        del_button.id = 'extra_substance_button_' + this.extra_substance;
        del_button.classList.add('btn', 'btn-danger', 'btn-circle', 'btn-sm');
        trash_icon.classList.add('fas', 'fa-trash');
        
        // Append to div
        del_button.appendChild(trash_icon);
        div.appendChild(input);
        div.appendChild(del_button);

        // Delete listener
        del_button.addEventListener('click', (event) => {
            event.preventDefault();
            div.remove();
            new_dropdown.remove();
            this.extra_substance -= 1;
        });

        molar_compositions_div.appendChild(div);
    }

    addBinaryInteraction() {
        const add_btn = document.getElementById('add_kij');
        const del_btn = document.getElementById('del_kij');
        const div = document.getElementById('binary_interaction_input');

        add_btn.classList.add('d-none');
        del_btn.classList.remove('d-none');

        this.createKijTable(div);
    }

    deleteBinaryInteraction() {
        const add_btn = document.getElementById('add_kij');
        const del_btn = document.getElementById('del_kij');
        const kij_table = document.getElementById('kij_table');

        add_btn.classList.remove('d-none');
        del_btn.classList.add('d-none');
        kij_table.remove();
    }

    createKijTable(div) {
        const table = document.createElement('table');
        const table_body = document.createElement('tbody');
        const header_row = document.createElement('tr');
        table.id = 'kij_table';

        let th;
        for (let i = 0; i <= this.extra_substance; i++) {
            th = document.createElement('th');
            th.innerText = i;
            header_row.appendChild(th);
        }

        table_body.appendChild(header_row);

        for (let i = 1; i <= this.extra_substance; i++) {
            let tr = document.createElement('tr');
            let td = document.createElement('td');
            td.innerText = i;
            tr.appendChild(td);
            for (let j = 1; j <= this.extra_substance; j++) {
                let td_input = document.createElement('td');
                let input = document.createElement('input');
                input.classList.add('form-control');
                input.setAttribute("type", "number");
                input.setAttribute("name", "kij[]");
                input.value = '0.0';

                if (i==j) {
                    input.disabled = true;
                }

                td_input.appendChild(input);
                tr.appendChild(td_input);
            }
            table_body.appendChild(tr);
        }

        table.appendChild(table_body);
        div.appendChild(table);
    }

    async configureButtons() {
        const liq_phase = document.getElementById('liq_phase').checked;
        const vap_phase = document.getElementById('vap_phase').checked;

        this.setAvailableButtons(liq_phase, vap_phase);
    }

    setAvailableButtons(liq, vap, all=false) {
        const bubble_t_btn = document.getElementById('bubble_temperature');
        const bubble_p_btn = document.getElementById('bubble_pressure');
        const dew_t_btn = document.getElementById('dew_temperature');
        const dew_p_btn = document.getElementById('dew_pressure');

        if (all){
            dew_t_btn.disabled = true;
            dew_p_btn.disabled = true;
            bubble_t_btn.disabled = true;
            bubble_p_btn.disabled = true;
        } else if (liq) {
            dew_t_btn.disabled = true;
            dew_p_btn.disabled = true;
            bubble_t_btn.disabled = false;
            bubble_p_btn.disabled = false;
        } else if (vap) {
            dew_t_btn.disabled = false;
            dew_p_btn.disabled = false;
            bubble_t_btn.disabled = true;
            bubble_p_btn.disabled = true;
        } else {
            dew_t_btn.disabled = true;
            dew_p_btn.disabled = true;
            bubble_t_btn.disabled = true;
            bubble_p_btn.disabled = true;
        }
    }

    getInputData() {
        const temperature = document.getElementById('temperature').value;
        const pressure = document.getElementById('pressure').value;
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;
        const liq_phase = document.getElementById('liq_phase').checked;
        const vap_phase = document.getElementById('vap_phase').checked;
        const substances_form = document.getElementsByName('substances[]');
        const compositions_form = document.getElementsByName('molar_compositions[]');
        const kij_form = document.getElementsByName('kij[]');
        
        let phase;
        if (liq_phase) {
            phase = 'liq';
        } else if (vap_phase) {
            phase = 'vap';
        }

        let substances = [];
        let compositions = [];
        let kij = [];

        for(let element of substances_form.values()) { 
            substances.push(element.value);
        }

        for(let element of compositions_form.values()) {
            compositions.push(element.value);
        }

        for(let element of kij_form.values()) {
            kij.push(element.value);
        }

        return {temperature, pressure, eos, alfa, phase, substances, compositions, kij};
    }

    async bubble_temperature() {
        const url = API_URL + '/mixture/bubble-temperature';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        if (body.message) {
            this.renderWarning(body.message);
            return;
        }

        this.renderSolution(body);
    }

    async bubble_pressure() {
        const url = API_URL + '/mixture/bubble-pressure';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        if (body.message) {
            this.renderWarning(body.message);
            return;
        }

        this.renderSolution(body);
    }

    async dew_temperature() {
        const url = API_URL + '/mixture/dew-temperature';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        if (body.message) {
            this.renderWarning(body.message);
            return;
        }

        this.renderSolution(body);
    }

    async dew_pressure() {
        const url = API_URL + '/mixture/dew-pressure';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        if (body.message) {
            this.renderWarning(body.message);
            return;
        }

        this.renderSolution(body);
    }

    renderSolution(body) {
        const liquid_solution = body.liquid;
        const liquid_div = document.getElementById('liq_solution');
        const liquid_title = document.getElementById('liq_solution_title');
        const liquid_card = document.getElementById('liq_solution_card');
        const liquid_table = document.getElementById('Liquid_solution_table');
        const vapor_table = document.getElementById('Vapor_solution_table');
        const vapor_card = document.getElementById('vap_solution_card');
        const vapor_title = document.getElementById('vap_solution_title');
        const vapor_solution = body.vapor;
        const vapor_div = document.getElementById('vap_solution');
        const warning_div = document.getElementById('warning');

        if (!warning_div.classList.contains('d-none')) {
            warning_div.classList.add('d-none');
        }

        console.log(liquid_table || vapor_table);
        
        if(liquid_table || vapor_table) {
            console.log('here');
            liquid_table.remove();
            vapor_table.remove();
        }

        this.createSolutionTable(liquid_div, liquid_solution);
        this.createSolutionTable(vapor_div, vapor_solution);
        liquid_title.innerText = liquid_solution.phase + ' phase at ' + liquid_solution.temperature + ' K and ' + liquid_solution.pressure + ' bar';
        vapor_title.innerText = vapor_solution.phase + ' phase at ' + vapor_solution.temperature + ' K and ' + vapor_solution.pressure + ' bar';
        liquid_card.classList.remove('d-none');
        vapor_card.classList.remove('d-none');
    }

    createSolutionTable(div, solution) {
        const table = document.createElement('table');
        const table_body = document.createElement('tbody');
        const header_row = document.createElement('tr');
        table.id = solution.phase + '_solution_table';
        table.classList.add('table', 'table-bordered');
        table.setAttribute('width','100%');
        table.setAttribute('cellspacing', '0');

        const headers = ['Substances', 'Composition', 'Enthalpy [J/mol]', 'Entropy [J/mol.K]', 'Fugacity'];
        let th;
        for (const title in headers) {
            th = document.createElement('th');
            th.innerText = headers[title];
            header_row.appendChild(th);
        }

        table_body.appendChild(header_row);

        for (let i = 0; i < solution.substances.length; i++) {
            let tr = document.createElement('tr');
            let substance_td = document.createElement('td');
            let composition_td = document.createElement('td');
            let enthalpy_td = document.createElement('td');
            let entropy_td = document.createElement('td');
            let fugacity_td = document.createElement('td');

            substance_td.innerText = solution.substances[i];
            composition_td.innerText = solution.compositions[i];
            enthalpy_td.innerText = solution.enthalpy[i];
            entropy_td.innerText = solution.entropy[i];
            fugacity_td.innerText = solution.fugacity[i];

            tr.appendChild(substance_td);
            tr.appendChild(composition_td);
            tr.appendChild(enthalpy_td);
            tr.appendChild(entropy_td);
            tr.appendChild(fugacity_td);

            table_body.appendChild(tr);
        }

        table.appendChild(table_body);
        div.appendChild(table);
    }

    renderWarning(message) {
        const warning_div = document.getElementById('warning');
        const warninig_text = document.getElementById('warnin_text');
        const liq_solution = document.getElementById('liq_solution_card');
        const vap_solution = document.getElementById('vap_solution_card');
        
        if (!liq_solution.classList.contains('d-none')) {
            liq_solution.classList.add('d-none');
        }

        if (!vap_solution.classList.contains('d-none')) {
            vap_solution.classList.add('d-none');
        }

        warning_div.classList.remove('d-none');
        warninig_text.innerText = message;
    }

    configureAjaxCalls() {
        document.getElementById('add_substance').addEventListener('click', this.addSubstance.bind(this));
        document.getElementById('add_kij').addEventListener('click', this.addBinaryInteraction.bind(this));
        document.getElementById('del_kij').addEventListener('click', this.deleteBinaryInteraction.bind(this));
        document.getElementById('liq_phase').addEventListener('change', this.configureButtons.bind(this));
        document.getElementById('vap_phase').addEventListener('change', this.configureButtons.bind(this));
        document.getElementById('bubble_temperature').addEventListener('click', this.bubble_temperature.bind(this));
        document.getElementById('bubble_pressure').addEventListener('click', this.bubble_pressure.bind(this));
        document.getElementById('dew_temperature').addEventListener('click', this.dew_temperature.bind(this));
        document.getElementById('dew_pressure').addEventListener('click', this.dew_pressure.bind(this));
    }
}

export const mixture = new Mixture();
