'use strict';

import { API_URL } from "../../constants";
import Plotly from 'plotly.js-basic-dist-min';

class PressureVolumeDiagram {

    get_substance() {
        const substance = document.getElementsByName('substance');
        let selected_substance = { 'id': null, 'name': null };
        for (let i = 0; i < substance.length; i++) {
            if (substance[i].checked) {
                selected_substance.id = substance[i].value;
                selected_substance.name = substance[i].getAttribute('data-name');
            }
        }

        return selected_substance;
    }

    async render() {
        const selected_substance = this.get_substance();
        const temperature = document.getElementById('temperature').value;
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;
        const url = API_URL + '/pure-substance/pv';
        const data = { "selected_substance": selected_substance.id, temperature, eos, alfa };
        const solution_ele = document.getElementById('solution');
        const card_title = document.getElementById('solution_title');
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        card_title.innerText = 'Pressure - Volume diagram for ' + selected_substance.name + ' at ' + data.temperature + ' K';
        const pv_line = {
            x: body.volumes,
            y: body.pressures,
            mode: 'markers',
            type: 'scatter'
        };

        const points = [pv_line];
        const layout = {
            title: 'PV diagram for ' + selected_substance.name + ' at ' + data.temperature + ' K',
            xaxis: {
                title: {
                    text: 'log(volume) [log(cm3)]',
                },
                type: 'log',
                automargin: true
            },
            yaxis: {
                title: {
                    text: 'Pressure [bar]',
                }
            }
        }

        const config = { responsive: true }

        Plotly.newPlot('pure_pv', points, layout, config);
        solution_ele.classList.remove('d-none');
    }

    async download_pv_csv() {
        const selected_substance = this.get_substance();
        const temperature = document.getElementById('temperature').value;
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;
        const url = API_URL + '/pure-substance/pv';
        const data = { "selected_substance": selected_substance.id, temperature, eos, alfa };
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        let csvContent = "data:text/csv;charset=utf-8,pressure,volume\r\n";

        for (let i = 0; i < body.pressures.length; i++) {
            const pressure = body.pressures[i];
            const volume = body.volumes[i];
            csvContent += pressure + ',' + volume + "\r\n";
        }

        const encodedUri = encodeURI(csvContent);
        const link = document.createElement("a");
        link.setAttribute("href", encodedUri);
        link.setAttribute("download", selected_substance.name + ' at ' + data.temperature + ' K.csv');
        document.body.appendChild(link); // Required for FF

        link.click();
    }



    configureAjaxCalls() {
        document.getElementById('draw_btn').addEventListener('click', this.render.bind(this));
        document.getElementById('download_pv_csv').addEventListener('click', this.download_pv_csv.bind(this));
    }

}

export const pressure_volume_diagram = new PressureVolumeDiagram();
