'use strict';

import { API_URL } from "../../constants";
import Plotly from 'plotly.js-gl3d-dist-min';

class PressureVolumeTemperatureDiagram {

    get_substance() {
        const substance = document.getElementsByName('substance');
        let selected_substance = { 'id': null, 'name': null };
        for (let i = 0; i < substance.length; i++) {
            if (substance[i].checked) {
                selected_substance.id = substance[i].value;
                selected_substance.name = substance[i].getAttribute('data-name');
            }
        }

        return selected_substance;
    }

    async render() {
        const selected_substance = this.get_substance();
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;
        const url = API_URL + '/pure-substance/pvt';
        const data = { "selected_substance": selected_substance.id, eos, alfa };
        const solution_ele = document.getElementById('solution');
        const card_title = document.getElementById('solution_title');
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        card_title.innerText = 'Pressure - Volume - Temperature diagram for ' + selected_substance.name;
        const pvt_surface = [{
            x: body.volumes,
            y: body.temperatures,
            z: body.pressures,
            type: 'mesh3d'
        }];

        const layout = {
            title: 'PVT diagram for ' + selected_substance.name,
            scene: {
                xaxis: {
                    title: '[log(cm3)]',
                    backgroundcolor: "rgb(200, 200, 230)",
                    gridcolor: "rgb(255, 255, 255)",
                    showbackground: true,
                    zerolinecolor: "rgb(255, 255, 255)",
                    type: 'log'
                },
                yaxis: {
                    title: '[K]',
                    backgroundcolor: "rgb(230, 200,230)",
                    gridcolor: "rgb(255, 255, 255)",
                    showbackground: true,
                    zerolinecolor: "rgb(255, 255, 255)"
                },
                zaxis: {
                    title: '[bar]',
                    backgroundcolor: "rgb(230, 230,200)",
                    gridcolor: "rgb(255, 255, 255)",
                    showbackground: true,
                    zerolinecolor: "rgb(255, 255, 255)"
                },
            }
        }

        const config = { responsive: true }

        Plotly.newPlot('pure_pvt', pvt_surface, layout, config);
        solution_ele.classList.remove('d-none');
    }

    configureAjaxCalls() {
        document.getElementById('draw_btn').addEventListener('click', this.render.bind(this));
    }

}

export const pressure_volume_temperature_diagram = new PressureVolumeTemperatureDiagram();
