'use strict';

import { API_URL } from "../../constants";

class PureSubstance {

    get_substance() {
        const substance = document.getElementsByName('substance');
        let selected_substance;
        for (let i = 0; i < substance.length; i++) {
            if (substance[i].checked) {
                selected_substance = substance[i].value;
            }
        }

        return selected_substance;
    }

    render_solution(data) {
        const solution_ele = document.getElementById('solution');
        const title = document.getElementById('solution_title');
        const vap_fugacity_ele = document.getElementById('vap_fugacity');
        const vap_enthalpy_ele = document.getElementById('enthalpy_vap');
        const vap_entropy_ele = document.getElementById('entropy_vap');
        const vap_z_ele = document.getElementById('z_vap');
        const liq_fugacity_ele = document.getElementById('liq_fugacity');
        const liq_enthalpy_ele = document.getElementById('enthalpy_liq');
        const liq_entropy_ele = document.getElementById('entropy_liq');
        const liq_z_ele = document.getElementById('z_liq');

        title.innerText = 'Solution at ' + data.temperature + ' K ' + 'and ' + data.pressure + ' bar';
        vap_fugacity_ele.innerText = data.vap_fugacity;
        vap_enthalpy_ele.innerText = data.enthalpy_vap;
        vap_entropy_ele.innerText = data.entropy_vap;
        vap_z_ele.innerText = data.z_vap;
        liq_fugacity_ele.innerText = data.liq_fugacity;
        liq_enthalpy_ele.innerText = data.enthalpy_liq;
        liq_entropy_ele.innerText = data.entropy_liq;
        liq_z_ele.innerText = data.z_liq;
        solution_ele.classList.remove('d-none');
        return 0;
    }

    async solve_eos() {
        const selected_substance = this.get_substance();
        const temperature = document.getElementById('temperature').value;
        const pressure = document.getElementById('pressure').value;
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;
        const url = API_URL + '/pure-substance/solve';
        const data = { selected_substance, temperature, pressure, eos, alfa };

        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        this.render_solution(body);
    }

    async solve_for_temperature() {
        const selected_substance = this.get_substance();
        const temperature = document.getElementById('temperature').value;
        const pressure = document.getElementById('pressure').value;
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;

        const url = API_URL + '/pure-substance/equilibrium';
        const data = {
            "selected_substance": selected_substance,
            "temperature": temperature,
            "pressure": pressure,
            "eos": eos,
            "alfa": alfa,
            "equilirbium": "temperature"
        }

        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json(); 
        this.render_solution(body);

    }

    async solve_for_pressure() {
        const selected_substance = this.get_substance();
        const temperature = document.getElementById('temperature').value;
        const pressure = document.getElementById('pressure').value;
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;

        const url = API_URL + '/pure-substance/equilibrium';
        const data = {
            "selected_substance": selected_substance,
            "temperature": temperature,
            "pressure": pressure,
            "eos": eos,
            "alfa": alfa,
            "equilirbium": "pressure"
        }

        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json(); 
        this.render_solution(body);
    }

    configureAjaxCalls() {
        document.getElementById('calculate_btn').addEventListener('click', this.solve_eos.bind(this));
        document.getElementById('solve_for_temperature').addEventListener('click', this.solve_for_temperature.bind(this));
        document.getElementById('solve_for_pressure').addEventListener('click', this.solve_for_pressure.bind(this));
    }

}

export const pure_substance = new PureSubstance();
