'use-strict';

import './bootstrap'
import { pure_substance } from './components/pureSubstance/PureSubstance';
import { pressure_volume_diagram } from './components/pureSubstance/PressureVolumeDiagram';
import { temperature_volume_diagram } from './components/pureSubstance/TemperatureVolumeDiagram';
import { pressure_volume_temperature_diagram } from './components/pureSubstance/PressureVolumeTemperatureDiagram';
import { binary_mixture } from './components/Binary/BinaryMixture';
import { pressure_composition_diagram } from './components/Binary/PressureCompositionDiagram';
import { temperature_composition_diagram } from './components/Binary/TemperatureCompositionDiagram';
import { mixture } from './components/Mixture/Mixture';
import { pressure_temperature_diagram } from './components/Mixture/PressureTemperatureDiagram';

(function () {

    // Toggle the side navigation
    $("#sidebarToggle, #sidebarToggleTop").on('click', function (e) {
        $("body").toggleClass("sidebar-toggled");
        $(".sidebar").toggleClass("toggled");
        if ($(".sidebar").hasClass("toggled")) {
            $('.sidebar .collapse').collapse('hide');
        };
    });

    // Close any open menu accordions when window is resized below 768px
    $(window).resize(function () {
        if ($(window).width() < 768) {
            $('.sidebar .collapse').collapse('hide');
        };
    });

    // Prevent the content wrapper from scrolling when the fixed side navigation hovered over
    $('body.fixed-nav .sidebar').on('mousewheel DOMMouseScroll wheel', function (e) {
        if ($(window).width() > 768) {
            var e0 = e.originalEvent,
                delta = e0.wheelDelta || -e0.detail;
            this.scrollTop += (delta < 0 ? 1 : -1) * 30;
            e.preventDefault();
        }
    });

    // Scroll to top button appear
    $(document).on('scroll', function () {
        var scrollDistance = $(this).scrollTop();
        if (scrollDistance > 100) {
            $('.scroll-to-top').fadeIn();
        } else {
            $('.scroll-to-top').fadeOut();
        }
    });

    // Smooth scrolling using jQuery easing
    $(document).on('click', 'a.scroll-to-top', function (e) {
        var $anchor = $(this);
        $('html, body').stop().animate({
            scrollTop: ($($anchor.attr('href')).offset().top)
        }, 1000, 'easeInOutExpo');
        e.preventDefault();
    });

    if (document.getElementById('dashboard')) {
        $('#dataTable').DataTable();
    }

    if (document.getElementById('pure_substance_equilibrium')) {
        pure_substance.configureAjaxCalls();
        $('#dataTable').DataTable();
    }

    if (document.getElementById('pure_substance_pv_diagram')) {
        pressure_volume_diagram.configureAjaxCalls();
        $('#dataTable').DataTable();
    }

    if (document.getElementById('pure_substance_tv_diagram')) {
        temperature_volume_diagram.configureAjaxCalls();
        $('#dataTable').DataTable();
    }

    if (document.getElementById('pure_substance_pvt_diagram')) {
        pressure_volume_temperature_diagram.configureAjaxCalls();
        $('#dataTable').DataTable();
    }

    if (document.getElementById('binary_equilibrium')) {
        $('.binary-first-select2').select2({
            theme:"bootstrap4"
        });
        $('.binary-second-select2').select2({
            theme:"bootstrap4"
        });
        binary_mixture.configureAjaxCalls();
    }

    if (document.getElementById('binary_pxy')) {
        $('.binary-first-select2').select2({
            theme:"bootstrap4"
        });
        $('.binary-second-select2').select2({
            theme:"bootstrap4"
        });

        pressure_composition_diagram.configureAjaxCalls();
    }

    if (document.getElementById('binary_txy')) {
        $('.binary-first-select2').select2({
            theme:"bootstrap4"
        });
        $('.binary-second-select2').select2({
            theme:"bootstrap4"
        });

        temperature_composition_diagram.configureAjaxCalls();
    }

    if (document.getElementById('mixture_equilibrium')) {
        $('.mixture-select2').select2({
            theme:"bootstrap4"
        });

        mixture.configureAjaxCalls();
    }

    if (document.getElementById('mixture_pressure_temperature_diagram')) {
        $('.mixture-select2').select2({
            theme:"bootstrap4"
        });

        pressure_temperature_diagram.configureAjaxCalls();
    }

})(); 
